/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import Applet.AppletMathGraph32;
import objets.CValeurDynamique;
import types.FonteNom;

@SuppressWarnings("serial")
public class ModificationFormuleDlg extends MtgDlg implements ActionListener, MouseListener {
	@SuppressWarnings("unused")
	private int indiceMaxi;         // L'indice maxi dans la liste d'objets du dernier objet qui puiss
  // figurer dans le combo
	private JLabel label;
	private ListeCalculsOuFonctionsModifiablesParMenu listeVal;
	private JButton btnModifier, btnFermer;
	private JTextArea info;

  public ModificationFormuleDlg(AppletMathGraph32 appletInit, int indiceMaxiInit) {
    super(getString("ch89"),true, appletInit);
    indiceMaxi = indiceMaxiInit;
    label = new JLabel(getString("ch90"));
    btnModifier = new JButton(getString("btnModifier"));
    btnFermer = new JButton(getString("btnFermer"));
    btnFermer.setMaximumSize(btnModifier.getPreferredSize());
    info = new JTextArea();
    info.setFont(FonteNom.getFonteNom(2, FonteNom.Normal));    
    JScrollPane scrollPane = new JScrollPane(info, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    info.setEditable(false);
    info.setColumns(20);
    info.setEditable(false);
    info.setLineWrap(false);
    info.setRows(4);
    scrollPane.setPreferredSize(new Dimension(450, 90));
    // Modification version 4.6
    // listeVal = new ListeCalculsOuFonctionsModifiablesParMenu(appletInit.listeConstruction, indiceMaxiInit, false);
    listeVal = new ListeCalculsOuFonctionsModifiablesParMenu(appletInit.listePourConstruction(), indiceMaxiInit, false);
    JPanel pane = new JPanel();
    GridBagLayout gridBag = new GridBagLayout();
    pane.setLayout(gridBag);
    GridBagConstraints constraints = new GridBagConstraints();
    setConstraints(constraints, 0, 0, 1, 1, 80, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    constraints.insets.top = 4;
    constraints.insets.bottom = 4;
    constraints.insets.left = 5;
    constraints.insets.right = 5;

    gridBag.setConstraints(label, constraints);
    pane.add(label);
    constraints.insets.top = 0;
    constraints.insets.bottom = 5;    
    setConstraints(constraints, 0, 1, 1, 1, 0, 70, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    JScrollPane scrollList = new JScrollPane(listeVal, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    
    //constraints.insets.bottom = 60;
    gridBag.setConstraints(scrollList, constraints);
    pane.add(scrollList);
    
    
    setConstraints(constraints, 0, 2, 2, 1, 100, 20, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER);
    gridBag.setConstraints(scrollPane, constraints);
    pane.add(scrollPane);
    
    constraints.insets.bottom = 0;
    
    JPanel paneDroit = new JPanel(); // Pour mettre les deux boutons en haut et  droite
    paneDroit.setLayout(new BoxLayout(paneDroit,BoxLayout.Y_AXIS));
    paneDroit.add(btnModifier);
    paneDroit.add(Box.createRigidArea(new Dimension(0,5)));    
    paneDroit.add(btnFermer);
    setConstraints(constraints, 1, 1, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.NORTHEAST);
    gridBag.setConstraints(paneDroit, constraints);
    pane.add(paneDroit);
    
    setContentPane(pane);
    btnModifier.addActionListener(this);
    btnFermer.addActionListener(this);
    listeVal.addMouseListener(this);
    listeVal.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent evt) {
      	listValSelectionChanged();
      }
    });
    
    getRootPane().setDefaultButton(btnFermer);      
    listValSelectionChanged();
    pack();  
  }

  public void actionPerformed(ActionEvent evt) {
    if (evt.getSource()== btnModifier) {
      actionButtonModifier();
    }
    else {
      if (evt.getSource()== btnFermer) {
        dispose();
      }
    }   
  }
  
  public void listValSelectionChanged() {
  	int i = listeVal.getSelectedIndex();
  	if (i != -1) {
  		info.setText(listeVal.pointeurs.get(i).infoPourHistorique());
  		info.setCaretPosition(0);
  	}
  }

  public void actionButtonModifier() {
  	int i = listeVal.getSelectedIndex();
  	if (i != -1) {
  		CValeurDynamique val = listeVal.pointeurs.get(i);
      if (val.modifiableParMenu()) {
      	MtgDlg dlg = val.dialogueModification(applet);
      	dlg.setLocationRelativeTo(this);
      	dlg.setVisible(true);
      	if (dlg.retourOK()) {
      		info.setText(listeVal.pointeurs.get(i).infoPourHistorique());
    		  applet.listePr.metAJourObjetsDependantDe(val);      		
      		applet.repaintFigureAvecCalcul();
      		listeVal.metAJourListe();
      		listValSelectionChanged();
      		/* Pas pour l'applet
      		if (val.getNatureCalcul() == NatureCalcul.NVariable)
      			applet.metAJourDialogueAffichageVariables((CVariableBornee) val);
      		frame.cadreActif().videListesPourConstructions();  // On remet  jour une construction ventuellement commence
      		*/
      	}
      }
  	}
  }

  public void mouseClicked(MouseEvent evt) {
    if (evt.getClickCount() == 2) {
      actionButtonModifier();
    }
  }

  public void mouseEntered(MouseEvent evt) {
  }

  public void mouseExited(MouseEvent evt) {
  }

  public void mousePressed(MouseEvent evt) {
  }

  public void mouseReleased(MouseEvent evt) {
  }	
}
