/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 *
 * @author  Yves
 * @version 
 */
package objets;

import java.io.*;
import java.awt.geom.*;
import java.awt.Color;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CCercleImage extends CCercle {
	private CCercle antecedent;
	private CTransformation transformation;
  private Point2D.Double point1 = new Point2D.Double();

  /** Creates new CCercleImage */
  public CCercleImage() {
  }

  // Constructeur pour clonage
  public CCercleImage(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean masqueInit, int styleInit,
        CCercle antecedentInit, CTransformation transformationInit) {
      super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit);
      antecedent = antecedentInit;
      transformation = transformationInit;
  }  

  /** Fonction devant tre redfinie pour tous les objets images par une transformation
   *  Sera utilise lorsqu'on modifie une image par une transformation pour que cette image
   *  soit associe  une autre transformation lrsque d'autres objets taient des images par
   *  cette mme transformation
   */
  public void associeATransformation(CTransformation trans) {
  	transformation = trans;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(antecedent);
    int ind2 = listeSource.indexOf(transformation);
    int ind3 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CCercleImage(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, masque, style, (CCercle)(listeCible.get(ind1, CCercle.class)),
        (CTransformation)(listeCible.get(ind2, CTransformation.class)));
    return ptelb;
  }  

  @Override
  public String infoPourHistorique() {
    return getString("chinfo35") + " " + transformation.designationTransf()
            + transformation.complementPourHist();
  }

  @Override
  public void ajouteAntecedents(CListeObjets liste) {
    liste.add(antecedent);
    transformation.AjouteElementsVisuelsPourHist(liste);
  } 
  
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || antecedent.dependDe(p) || transformation.dependDe(p));
  }

  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || antecedent.dependDePourBoucle(p) || transformation.dependDePourBoucle(p);
  }

  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
		existe = transformation.existe && antecedent.existe;
		if (!existe) return;
		transformation.transformePoint(antecedent.centre_x, antecedent.centre_y, point1);
    centre_x = point1.x;
    centre_y = point1.y;
		rayon = transformation.transformeLongueur(antecedent.rayon);
		super.positionne(infoRandom, dimfen);
  }

  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne.
  **/
  public boolean confonduAvec(CElementBase p) {
    CCercleImage pt;
    if (p.getClass() == this.getClass()) {
      pt = (CCercleImage) p;
      return (antecedent == pt.antecedent)
			&& transformation.confonduAvec(pt.transformation);
    }
    else return false;
  }  
  /**
   * fonction n'existant pas en C++ et renvoyant un pointeur vers l'antcdent direct de l'objet.
   * Si l'objet est un objet qui ne ncessite pas lors de sa cration la cration
   * d'objets supplmentaires, renvoie this et sinon envoi un pointeur sur le premier
   * de ces objets supplmentaires crs.
   * Par exemple, pour un point li  un bipoint (intersection de deux cercles
   * ou d'un cercle et une droite) renverra un pointeur sur le bipoint
   * cr avant ce CPointLieBipoint. Pour une objet image, renverra un pointeur
   * vers la transformation utilise etc ...
   */
  public CElementBase antecedentDirect() {
    return transformation;
  }
      
 public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    int ind2 = inps.readCInt();
    antecedent = (CCercle) (list.get(ind1,CCercle.class));
    transformation = (CTransformation) (list.get(ind2,CTransformation.class));
  }
  /* Enregistrement dans un flux java */  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    antecedent = (CCercle) (list.get(ind1,CCercle.class));
    transformation = (CTransformation) (list.get(ind2,CTransformation.class));
  }  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(antecedent);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(transformation);
    oups.writeInt(ind2);
  }
    
  @Override
  public boolean contientParDefinition(CPointAncetre po) {
  	if (po.estPointImage()) {
  		CPointImage ptim = (CPointImage) po;
  		return (transformation == ptim.transformation) && ptim.antecedent.appartientParDefinition(antecedent);
  	}
  	else return true;
  }
  
}