/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.geom.*;
import java.awt.Color;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
public class CDroiteImage extends CDroite {
	private CDroite antecedent;
	private CTransformation transformation;
  private Point2D.Double point1 = new Point2D.Double();

  /** Creates new CDroiteImage */
  public CDroiteImage() {
    super(); // Pour que lengthvecteur soit cr
  }
  
  /** Fonction devant tre redfinie pour tous les objets images par une transformation
   *  Sera utilise lorsqu'on modifie une image par une transformation pour que cette image
   *  soit associe  une autre transformation lrsque d'autres objets taient des images par
   *  cette mme transformation
   */
  public void associeATransformation(CTransformation trans) {
  	transformation = trans;
  }
  
  // Constructeur pour clonage
  public CDroiteImage(CListeObjets liste, CImplementationProto impProtoInit,boolean estElementFinalInit,
      Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit, 
      boolean masqueInit, String nomInit, int indiceTailleNomInit,int styleInit, 
      double abscisseNomInit , CDroite antecedentInit, CTransformation transformationInit) {
        super(liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
          dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit, styleInit, abscisseNomInit);
        antecedent = antecedentInit;
        transformation = transformationInit;
  }  

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(antecedent);
    int ind2 = listeSource.indexOf(transformation);
    int ind3 = listeSource.indexOf(impProto);     
    
    CElementBase ptelb = new CDroiteImage(listeCible,(CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y,
      masque, nom, indiceTailleNom, style, abscisseNom, (CDroite)(listeCible.get(ind1, CDroite.class)),
        (CTransformation)(listeCible.get(ind2, CTransformation.class)));
    return ptelb;
  }

  @Override
  public String infoPourHistorique() {
    return getString("chinfo34") + " " + transformation.designationTransf()
            + transformation.complementPourHist();
  }

  @Override
  public void ajouteAntecedents(CListeObjets liste) {
    liste.add(antecedent);
    transformation.AjouteElementsVisuelsPourHist(liste);
  } 

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || antecedent.dependDe(p) || transformation.dependDe(p));
  }

  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || antecedent.dependDePourBoucle(p) || transformation.dependDePourBoucle(p));
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
		existe = transformation.existe && antecedent.existe;
		if (!existe) return;
		transformation.transformePoint(antecedent.point_x, antecedent.point_y, point1);
    point_x = point1.x;
    point_y = point1.y;
		transformation.transformeVecteur(antecedent.vecteur, vecteur);
		super.positionne(infoRandom, dimfen);
  }

  
/** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne.
  **/
  @Override
  public boolean confonduAvec(CElementBase p) {
    CDroiteImage pt;
    if (p.getClass() == this.getClass()) {
      pt = (CDroiteImage) p;
      return (antecedent == pt.antecedent) && (transformation.confonduAvec(pt.transformation));
    }
    else return false;
  }  
  /**
   * fonction n'existant pas en C++ et renvoyant un pointeur vers l'antcdent direct de l'objet.
   * Si l'objet est un objet qui ne ncessite pas lors de sa cration la cration
   * d'objets supplmentaires, renvoie this et sinon envoi un pointeur sur le premier
   * de ces objets supplmentaires crs.
   * Par exemple, pour un point li  un bipoint (intersection de deux cercles
   * ou d'un cercle et une droite) renverra un pointeur sur le bipoint
   * cr avant ce CPointLieBipoint. Pour une objet image, renverra un pointeur
   * vers la transformation utilise etc ...
   */
  @Override
  public CElementBase antecedentDirect() {
    return transformation;
  }
    
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    int ind2 = inps.readCInt();
    antecedent = (CDroite) (list.get(ind1,CDroite.class));
    transformation = (CTransformation) (list.get(ind2,CTransformation.class));
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    antecedent = (CDroite) (list.get(ind1,CDroite.class));
    transformation = (CTransformation) (list.get(ind2,CTransformation.class));

  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(antecedent);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(transformation);
    oups.writeInt(ind2);
  }
  
  @Override
  public boolean contientParDefinition(CPointAncetre po) {
  	if (po.estPointImage()) {
  		CPointImage ptim = (CPointImage) po;
  		return (transformation == ptim.transformation) && ptim.antecedent.appartientParDefinition(antecedent);
  	}
  	else return false;
  }
  
  @Override
  public boolean estParalleleParDefinition(CDroiteAncetre dte){
  	return (dte == this) || 
  			(((transformation.natureTransformation() & (CTransformation.translation | CTransformation.homothetie | CTransformation.symetrieCentrale)) != 0)
  			&& (antecedent.estParalleleParDefinition(dte) || dte.estParalleleParDefinition(antecedent)));
  }
  
}
