/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;
import java.io.*;

import types.NatureCalcul;
import types.NatureObjet;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;


/**
 * Elment virtuel reprsentant un lment rel utilis dans un prototype
 * @author Yves
 */

public class CElementGenerique extends CElementBase {
  public String nom;
  public CElementBase elementAssocie;
  // Contrairement  la version C++, un lment gnrique doit contenir la nature de l'objet
  // par lequel il doit tre remplac dans le cas d'un lment source
  public long natureGraphique;
  public long natureCalcul;
  /** Creates a new instance of CElementGenerique */
  public CElementGenerique() {
  	super();
  	nom = "";
  	elementAssocie = null;
  	natureGraphique = 0;
  	natureCalcul = 0;
  }
  
  @Override  
  public void chargeDepuisFlux (MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
  // On ne charge pas vraiment les lments du prototype qui ne sert pas dans l'applet
    nom = inps.readCString();
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    nom = inps.readUTF();
    natureGraphique = nature(inps.readInt());
    natureCalcul = nature(inps.readInt());
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    oups.writeUTF(nom);
    oups.writeInt(indiceDeNature(natureGraphique));
    oups.writeInt(indiceDeNature(natureCalcul));
  }
  
  public void adapteNature(int indiceNatureC) {
  	natureGraphique = 0;
  	natureCalcul = 0;
  	switch (indiceNatureC) {
  	case 0 :
  		natureGraphique = NatureObjet.NPointBase;
  		break;
  	case 1 :
  		natureGraphique = NatureObjet.NPoint;
  		break;
  	case 2 :
  		natureGraphique = NatureObjet.NPointLie;
  		break;
  	case 3 :
  		natureGraphique = NatureObjet.NBarycentre;
  		break;
  	case 4 :
  		natureGraphique = NatureObjet.NSegment;
  		break;
  	case 5 :
  		natureGraphique = NatureObjet.NDroite;
  		break;
  	case 6 :
  		natureGraphique = NatureObjet.NVecteur;
  		break;
  	case 7 :
  		natureGraphique = NatureObjet.NDemiDroite;
  		break;
  	case 8 :
  		natureGraphique = NatureObjet.NCercle;
  		break;
  	case 9 :
  		natureGraphique = NatureObjet.NMarqueAngle;
  		break;
  	case 10 :
  		natureGraphique = NatureObjet.NValeurAffichee;
  		break;
  	case 11 :
  		natureGraphique = NatureObjet.NArc;
  		break;
  	case 12 :
  		natureGraphique = NatureObjet.NLieu;
  		break;
  	case 13 :
  	  // Attention : Dans la version java, le repre est  la fois graphique (car il doit tre affich) et non graphique
  	  // Il est considr comme non graphique car dans les constructions il est choisi en premier dans
  	  // les objets non graphiques
  		natureGraphique = NatureObjet.NRepere;
  		natureCalcul = NatureCalcul.NRepere;
  		break;
  	case 14 :
  		natureGraphique = NatureObjet.NPointDansRepere;
  		break;
  	case 15 :
  		natureGraphique = NatureObjet.NCommentaire;
  		break;
  	case 16 :
  		natureGraphique = NatureObjet.NPointLiePoint;
  		break;
  	case 17 :
  		natureGraphique = NatureObjet.NPolygone;
  		break;
  	case 18 :
  		natureGraphique = NatureObjet.NSurface;
  		break;
  	case 19 :
  		natureGraphique = NatureObjet.NObjetDuplique;
  		break;
  	case 20 :
  		natureGraphique = NatureObjet.NGrapheSuiteRec;
  		break;
  	case 21 :
  		natureGraphique = NatureObjet.NBipoint;
  		break;
  	case 22 :
  		natureGraphique = NatureObjet.NTransformation;
  		break;
  	case 23 :
  		natureGraphique = NatureObjet.NValeurAffichee;
  		break;
  	case 24 :
  		natureCalcul = NatureCalcul.NVariable;
  		break;
  	case 25 :
  		natureCalcul = NatureCalcul.NCalculReel;
  		break;
  	case 26 :
  		natureCalcul = NatureCalcul.NFonction;
  		break;
  	case 27 :
  		natureCalcul = NatureCalcul.NSuiteRecurrenteReelle;
  		break;
  	case 28 :
  		natureGraphique = NatureObjet.NMacro;
  		break;
  	case 29 :
  		natureGraphique = NatureObjet.NDemiPlan;
  		break;
  	case 30 :
  		natureCalcul = NatureCalcul.NAutreCalcul;
  		break;
  	case 31 :
  		natureGraphique = NatureObjet.NLieuDiscret;
  		break;
  	case 32 :
  		natureGraphique = NatureObjet.NLigneBrisee;
  		break;
  	case 33 :
  		natureGraphique = NatureObjet.NMarqueSegment;
  		break;
  	case 34 :
  		natureGraphique = NatureObjet.NImage;
  		break;
  	case 35 :
  		natureGraphique = NatureObjet.NLieuObjet;
  		break;
  	case 36 :
  		natureCalcul = NatureCalcul.NCalculComplexe;
  		break;
  	case 37 :
  		natureCalcul = NatureCalcul.NFonctionComplexe;
  		break;
  	case 38 :
  		natureGraphique = NatureObjet.NPointParAffixe;
  		break;
  	case 39 :
  		natureCalcul = NatureCalcul.NSuiteRecurrenteComplexe;
  		break;
  	case 40 :
  		natureCalcul = NatureCalcul.NMesureAffixe; //  revoir
  		break;
  	case 41 :
  		natureGraphique = NatureObjet.NGrapheSuiteRecComplexe;
  		break;	
  	}
  }
  
  @Override
  public long getNature() {
  	return natureGraphique;
  }  
  
  @Override
  public long getNatureCalcul() {
  	return natureCalcul;
  }
  
  public int indiceDeNature(long nature) {
  	if (nature == 0) return - 1;
  	int ind;
  	long nat = 1;
  	for (ind = 0; ind <= 63; ind++) {
  		if ((nature & nat) != 0) break;
  		else nat = nat << 1;
  	}
  	return ind;
  }
  
  public long nature(int indiceNature) {
  	if (indiceNature == -1) return 0;
  	return 1L << indiceNature;
  }
}
