/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;

import java.awt.Color;
import types.DimensionFenetre;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;


public class CLieuObjetParVariable extends CLieuObjetAncetre {
  CVariableBornee  variableGeneratrice;
  
  /**
   * Creates new CLieuObjetParVariable
   */
  public CLieuObjetParVariable() {
  }
  
  // Constructeur pour clonage
  public CLieuObjetParVariable(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, CElementGraphique elementAssocieInit, CValeurAssocieeAVariable nombreTracesInit,
    CVariableBornee variableGeneratriceInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit,
      masqueInit, elementAssocieInit,  nombreTracesInit);
    variableGeneratrice = variableGeneratriceInit;
    // Rajout version 3.4.2
    // etablitListeElementsAncetres(); Inutile et provoquait un bug sous version 4.4 en appelant getClone
  }
  
  /**
   * A partir de la version 3.4.2, changement de version (version 1 -> 2) car on enregistre dans le flux la CSousListeObjetstres
   * contenant les pointeurs sur les lments anctres
   */
  @Override
  public int numeroVersion() {
  	return 2;
  }  
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(elementAssocie);
    int ind2 = listeSource.indexOf(variableGeneratrice);
    int ind3 = listeSource.indexOf(impProto);
    CValeurAssocieeAVariable nombreTracesClone = (CValeurAssocieeAVariable) nombreTraces.getClone(listeSource, listeCible);
    CElementBase ptelb = new CLieuObjetParVariable(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, masque, (CElementGraphique)(listeCible.get(ind1, CElementGraphique.class)), nombreTracesClone,
      // (CVariableBornee)(listeCible.get(ind2))); // A revoir version 4.7.7 dernier paramtre
      (CVariableBornee)(listeCible.get(ind2,CVariableBornee.class)));
    if (listeCible.getClass() != CPrototype.class) {    
	    ((CLieuObjetParVariable)ptelb).listeElementsAncetres.setImage(listeElementsAncetres, listeProprietaire,
	      ptelb.listeProprietaire);
	  // Modification version 2.0
    // La ligne suivante ne doit pas tre appele quand on implmente un prototype. Cela sera fait par metAJour()
    // Comme le nombre d'objets est dynamique on ne peut pas savoir au chargement combien il y en a.
    // if (listeSource.getClass() != CPrototype.class) ((CLieuObjetParVariable)ptelb).listeCopiesObjet.ajouteClonesDe
    //	(((CLieuObjetParVariable)ptelb).elementAssocie, listeCopiesObjet.size());
    }
    return ptelb;
  }
  
  @Override
  public String infoPourHistorique() {
    return getString("chinfo142") + " " + variableGeneratrice.nom();
  }
  
  /**
   * Fonction n'existant pas en C++ et charge de mettre  jour l'objet quand un lment dont il dpend a t modifi.
   * Devra tre appel par les lieux de points et lieux d'objets pour remettre  jour les liste qu'ils utilisent
   * de faon interne
   * Pour les lieux d'objets il faut aussi recrer la liste d'objets clones servant pour la trace
   */
  @Override
  public void metAJour() {
  	super.metAJour();
  	// Bug corrig version 4.0 : il faut mettre  jour la liste d'lments anctres car metAJour est appel quand
  	// on modifie un obejt dont dpend cet objet
  	// if (nVersion == 1) etablitListeElementsAncetres();
  	etablitListeElementsAncetres();
  }
   
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    double valeur, valeurMinimale, valeurMaximale, valeurInitiale, pas;
    super.positionne(infoRandom, dimfen);
    existe = existe && (nombreObjetsPourLieuObjet() <= nombreMaxiPourLieuObjet);
    double dnbt = Math.round(nombreTraces.rendValeur(false));
    existe = existe && (dnbt>=2) && (dnbt<=nombreMaxiPourLieuObjet);
    if (!existe) return;
    int nbt = (int) dnbt;
    // Changement important de la version 2.5, le nombre de traces tant devenu dynamique
    if (nbt != listeCopiesObjet.size()) ajusteListeCopieObjets();
    valeurMinimale = variableGeneratrice.valeurMini;
    valeurMaximale = variableGeneratrice.valeurMaxi;
    valeurInitiale = variableGeneratrice.valeurActuelle;
    // Pour tenir compte des points lis lis  des lieux ferms
    pas = (valeurMaximale-valeurMinimale)/(nbt - 1);
    valeur = valeurMinimale;
    
    boolean elementAssocieMasque = elementAssocie.masque;
    elementAssocie.montre();
    existe = false;
    int nb = nbt - 1;
    for (int i = 0; i < nbt; i++) {
    	// Optimis version 4.0
    	valeur = (i == 0) ? valeurMinimale : ((i == nb) ? valeurMaximale: valeur + pas);
      variableGeneratrice.donneValeur(valeur);
      listeElementsAncetres.positionne(infoRandom, dimfen);
      existe = existe || elementAssocie.existe;
      CElementGraphique ptelg = (CElementGraphique)listeCopiesObjet.get(i);
      ptelg.setClone(elementAssocie);
      // Au cas o l'objet gnrateur du lieu serait cach
      ptelg.montre();
    }
    elementAssocie.masque = elementAssocieMasque;
    // On redonne  la variable gnratrice sa valeur initiale
    variableGeneratrice.donneValeur(valeurInitiale);
    listeElementsAncetres.positionne(infoRandom, dimfen);
  }
  
  @Override
  public boolean confonduAvec(CElementBase p) {
    CLieuObjetParVariable pt;
    if (p.getClass() == this.getClass()) {
      pt = (CLieuObjetParVariable) p;
  		return (variableGeneratrice == pt.variableGeneratrice)
			&& (elementAssocie == pt.elementAssocie)
			&& nombreTraces.confonduAvec(pt.nombreTraces);    
		}
    else return false;
  }  
  
  public void etablitListeElementsAncetres() {
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    if (listeProprietaire.getClass() != CPrototype.class) {
	    listeElementsAncetres.retireTout();
	    int indicePointATracer = listeProprietaire.indexOf(elementAssocie);
	    int indiceVariableGeneratrice = listeProprietaire.indexOf(variableGeneratrice);
	    listeElementsAncetres.add(variableGeneratrice);
      for (int i = indiceVariableGeneratrice + 1; i < indicePointATracer; i++) {
        CElementBase ptelb = (CElementBase)(listeProprietaire.get(i));
        if ((elementAssocie.dependDe(ptelb)) && (ptelb.dependDe(variableGeneratrice)))
          listeElementsAncetres.add(ptelb);
      }
      listeElementsAncetres.add(elementAssocie);
    }	
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    if (nVersion == 1) {
    	super.chargeDepuisFlux(inps, list);
      nVersion = numeroVersion(); // On remet tout  zro pour la version java
    }
    else super.chargeDepuisFluxNew(inps, list);
    int ind1 = inps.readCInt();
    variableGeneratrice = (CVariableBornee) (list.get(ind1,CVariableBornee.class));
    // Il faut prparer la liste des lments dont dpend le point  tracer
    // Version 1.9.5 : modifi pour optimisation
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    if (list.getClass() != CPrototype.class) {
      int indiceElementAssocie = list.indexOf(elementAssocie);
      for (int i = 0; i <= indiceElementAssocie; i++) {
        CElementBase ptelb = (CElementBase)(list.get(i));
        if ((elementAssocie.dependDe(ptelb)) && (ptelb.dependDe(variableGeneratrice)))
          listeElementsAncetres.add(ptelb);
      }
    }
    nVersion = numeroVersion(); // On remet tout  zro pour la version java
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    variableGeneratrice = (CVariableBornee) (list.get(ind1,CVariableBornee.class));
    // Il faut prparer la liste des lments dont dpend le point  tracer
    // Version 1.9.5 : modifi pour optimisation
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    // Modifi version 3.4.2
    /*
    if (list.getClass() != CPrototype.class) {
      int indiceElementAssocie = list.indexOf(elementAssocie);
      for (int i = 0; i <= indiceElementAssocie; i++) {
        CElementBase ptelb = (CElementBase)(list.get(i));
        if ((elementAssocie.dependDe(ptelb)) && (ptelb.dependDe(variableGeneratrice)))
          listeElementsAncetres.add(ptelb);
      }
    }
    */
    if (nVersion == 1) etablitListeElementsAncetres();
    else listeElementsAncetres.read(inps, list);
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(variableGeneratrice);
    oups.writeInt(ind1);
    listeElementsAncetres.write(oups, listeProprietaire);
  }
}
