/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

// Classe anctre des lignes brises et des polygones
import java.io.*;
import java.util.*;
import java.awt.*;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;


// Modification pour la version 2.5 : la classe n'est plus abstract car il
// faut pouvoir l'instancier dans CListeObjets.get(int indice, Class cla)
public class CLignePolygonaleAncetre extends CElementLigne {
  // collectionPoints est une liste chaine contenant des pointeurs sur les sommets de la ligne
  public Vector<CNoeudPointeurSurPoint> collectionPoints;
  // Membre rajout pour la version 1.9.5
  int nombrePoints;
  // Les coordonnes des sommets de la ligne
  int[] abscissesCoordonneesPoints;
  int[] ordonneesCoordonneesPoints;
  // Objets servant dans distancePoint()
  private CPointBase point1;
  private CPointBase point2;
  private CSegment segment;
  
  /** Creates new CLignePolygonaleAncetre
   *  Ne sert que pour le chargement via un flux */
  public CLignePolygonaleAncetre() {
  }
  
  public CLignePolygonaleAncetre(CListeObjets liste, Color couleurInit, int styleInit) {
  	super(liste, null, false, couleurInit, false, styleInit);
    point1 = new CPointBase();
    point2 = new CPointBase();
    segment = new CSegment(liste, point1, point2);
    collectionPoints = new Vector<CNoeudPointeurSurPoint>();
    // On ne peut pas  ce niveau crer les tableaux contenant les coordonnes car on ne sait pas combien le
    // polygone contient de sommets
  }
  
  /**
   * Construction  partir d'un vector de CNoeudPointeurSurPoint
   * @param liste
   * @param vecteur
   * @param couleurInit
   * @param styleInit
   */
  public CLignePolygonaleAncetre(CListeObjets liste, Vector<CNoeudPointeurSurPoint> vecteur, Color couleurInit, int styleInit) {
  	super(liste, null, false, couleurInit, false, styleInit);
    point1 = new CPointBase();
    point2 = new CPointBase();
    segment = new CSegment(liste, point1, point2);
    collectionPoints = vecteur;
    prepareTableauxCoordonnees();
  }
  
  // Constructeur pour clonage
  // Il faut cloner chaque lment de la liste de noeuds pondrs.
  public CLignePolygonaleAncetre(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, int styleInit, Vector<CNoeudPointeurSurPoint> collectionPointsInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit);
    point1 = new CPointBase();
    point2 = new CPointBase();
    segment = new CSegment(liste, point1, point2);
    collectionPoints = collectionPointsInit;
    abscissesCoordonneesPoints = new int[collectionPoints.size()];
    ordonneesCoordonneesPoints = new int[collectionPoints.size()];
  }
  
  public void prepareTableauxCoordonnees() {
  	nombrePoints = collectionPoints.size();
    abscissesCoordonneesPoints = new int[nombrePoints];
    ordonneesCoordonneesPoints = new int[nombrePoints]; 	
  }
  
  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    nombrePoints = ((CLignePolygonaleAncetre) ptel).nombrePoints;
    for (int i = 0; i < collectionPoints.size(); i ++) {
      abscissesCoordonneesPoints[i] = ((CLignePolygonaleAncetre) ptel).abscissesCoordonneesPoints[i];
      ordonneesCoordonneesPoints[i] = ((CLignePolygonaleAncetre) ptel).ordonneesCoordonneesPoints[i];
    }
  }
  
  
  // Version 4.7.6 : 0ptimis
  public boolean dependDePourBoucle(CElementBase p) {
    boolean resultat = (p == this);
    ListIterator<CNoeudPointeurSurPoint> li = collectionPoints.listIterator();
    while(li.hasNext() && !resultat) {
      CPointAncetre ptp = (CPointAncetre)((li.next())).pointeurSurPoint;
      resultat = resultat || ptp.dependDePourBoucle(p);
    }
    return resultat;
  }
  
  // Version 4.7.6 : Optimis
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    boolean resultat = super.dependDe(p);
    ListIterator<CNoeudPointeurSurPoint> li = collectionPoints.listIterator();
    while(li.hasNext() && !resultat) {
      CPointAncetre ptp = (CPointAncetre)(li.next()).pointeurSurPoint;
      resultat = resultat || ptp.dependDe(p);
    }
    return memoriseDependance(resultat);
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    // Rajout pour la version 1.9.5 car Trace ne doit plus faire rfrence  des pointeurs
    nombrePoints = collectionPoints.size();
    existe = true;
    int i = 0;
    ListIterator<CNoeudPointeurSurPoint> li = collectionPoints.listIterator();
    while(li.hasNext()) {
      CNoeudPointeurSurPoint noeud = li.next();
      CPointAncetre ptp = noeud.pointeurSurPoint;
      existe = existe && ptp.existe;
      if (!existe) return;
      existe = existe && !ptp.horsEcran;
      if (existe) {
        abscissesCoordonneesPoints[i] = (int) (Math.round(ptp.x));
        ordonneesCoordonneesPoints[i] = (int) (Math.round(ptp.y));
        i++;
      } else return;
    } // while
  }
  
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
    
    int styleTrait;
    if (masque) styleTrait = StyleTrait.traitPointille;
    else styleTrait = style;
    pDC.setStroke(Pinceau.CreePinceau(styleTrait));
    pDC.setColor(couleur);
    pDC.drawPolyline(abscissesCoordonneesPoints, ordonneesCoordonneesPoints, nombrePoints);
  }
  
  public String nomSommets() {
  	int lon = 0;
    ListIterator<CNoeudPointeurSurPoint> li = collectionPoints.listIterator();
    while(li.hasNext()) {
      CNoeudPointeurSurPoint noeud = li.next();
      CPointAncetre ptp = noeud.pointeurSurPoint;
      lon += ptp.nom.length();
    }
  	StringBuffer stb = new StringBuffer(lon);
    ListIterator<CNoeudPointeurSurPoint> li2 = collectionPoints.listIterator();
    while(li2.hasNext()) {
      CNoeudPointeurSurPoint noeud = li2.next();
      CPointAncetre ptp = noeud.pointeurSurPoint;
      stb.append(ptp.nom);
    }
    return stb.toString();
  }
  
  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    ListIterator<CNoeudPointeurSurPoint> li = collectionPoints.listIterator();
    while(li.hasNext()) {
      CNoeudPointeurSurPoint ptn = li.next();
      ptn.remplacePoint(ancienPoint, nouveauPoint);
    }
  }
  
  @Override
  public double distancePoint(double xp, double yp, boolean masquage) {
 	 if (!(existe) || (masquage && masque)) return -1;
 	 else return distancePointPourSurface(xp,yp,masquage);
 }
  
  /** distancePoint renvoie -1 dans le cas o le l'objet n'est pas dsignable */
  public double distancePointPourSurface(double xp, double yp, boolean masquage) {
    double dis;
    if (!existe) return -1;
    else {
      dis = Double.MAX_VALUE;
      for (int i = 0; i < nombrePoints - 1; i ++) {
        /* Modifi pour la version 1.9.5 pour qu'on puisse dsigner une trace d'objets
         * il ne faut plus utiliser de pointeurs dans distancePoint mais seulement des
         * valeurs calcules dans positionne()
        CPointAncetre pt1 = (CPointAncetre) ((CNoeudPointeurSurPoint)(collectionPoints.get(i))).pointeurSurPoint;
        CPointAncetre pt2 = (CPointAncetre) ((CNoeudPointeurSurPoint)(collectionPoints.get(i+1))).pointeurSurPoint;
        point1.placeEn(pt1);
        point2.placeEn(pt2);
         **/
        point1.placeEn(abscissesCoordonneesPoints[i], ordonneesCoordonneesPoints[i]);
        point2.placeEn(abscissesCoordonneesPoints[i+1], ordonneesCoordonneesPoints[i+1]);
        segment.positionne();
        double d = segment.distancePoint(xp, yp, true);
        if ((d != -1) && (d < dis)) dis = d;
      }
    }
    if (dis == Double.MAX_VALUE) dis = -1;
    return dis;
  }

  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    point1 = new CPointBase();
    point2 = new CPointBase();
    segment = new CSegment(list, point1, point2);
    collectionPoints = new Vector<CNoeudPointeurSurPoint>();
    int nombreObjets = inps.readCCount();
    for (int i = 0; i < nombreObjets; i++) {
      CNoeudPointeurSurPoint noeud = (CNoeudPointeurSurPoint) inps.readCObject(list);
      collectionPoints.add(noeud);
    }
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    if (list.getClass() != CPrototype.class) prepareTableauxCoordonnees();
  }
  
  // A t amlior par rapport  chargeDepuisFlux
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    point1 = new CPointBase();
    point2 = new CPointBase();
    segment = new CSegment(list, point1, point2);
    collectionPoints = new Vector<CNoeudPointeurSurPoint>();
    int nombreObjets = inps.readInt();
    for (int i = 0; i < nombreObjets; i++) {
      CNoeudPointeurSurPoint noeud = new CNoeudPointeurSurPoint();
      noeud.read(inps, list);
      collectionPoints.add(noeud);
    }
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    if (list.getClass() != CPrototype.class) prepareTableauxCoordonnees();
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeInt(collectionPoints.size());
    ListIterator<CNoeudPointeurSurPoint> li = collectionPoints.listIterator();
    while (li.hasNext()) {
      CNoeudPointeurSurPoint noeud = li.next();
      noeud.write(oups, list);
    }
  }
}
