/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.*;
import java.awt.geom.*;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;


public class CMarqueAngleDroit extends CMarqueAngleAncetre {
	private CDroitePerpendiculaire perp;
  private Point2D.Double intersection = new Point2D.Double(0,0);

  /** Creates new CMarqueAngleDroit */
  public CMarqueAngleDroit() {
  }
  
  /** Constructeur pour clonage */
  public CMarqueAngleDroit(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, int styleInit, int styleMarqueInit, double rayonInit,  CDroitePerpendiculaire perpInit) {
    super (liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit, styleMarqueInit, rayonInit);
    perp = perpInit;
  }
  
  /** Constructeur plus court pour objets visuels */
  public CMarqueAngleDroit(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    CDroitePerpendiculaire perpInit) {
    super(liste, impProtoInit, estElementFinalInit, Couleur.TraduitCouleur(1), false, StyleTrait.traitFin, 
    		StyleMarqueAngle.marqueSimple, 9);
    perp = perpInit;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(perp);
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CMarqueAngleDroit(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, masque, style, styleMarque, rayon, (CDroitePerpendiculaire)(listeCible.get(ind1, CDroitePerpendiculaire.class)));
    return ptelb;
  }
    
  public void ajouteAntecedents(CListeObjets liste) {
    liste.add(perp);
  }    
  
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
		return memoriseDependance(super.dependDe(p) || perp.dependDe(p));
  }

  @Override
  public boolean dependDePourBoucle(CElementBase p) {
		return ((p == this) || perp.dependDePourBoucle(p));
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
		existe = perp.existe;
		if (!(existe)) return;
		Kernel.intersectionDroitesSecantes(perp.A.x,perp.A.y,perp.vecteur,
			perp.D.point_x,perp.D.point_y,perp.D.vecteur,intersection);
		centre_x = intersection.x;
		centre_y = intersection.y;
    perp.vecteur.setCopy(u);
		perp.D.vecteur.setCopy(v);
		ang1rad = u.angleRad();
		ang2rad = v.angleRad();
		prepareAngleDroit();
		super.positionne(infoRandom, dimfen);
  }

  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
    
    int styleTrait;
    if (masque) styleTrait = StyleTrait.traitPointille;
    else styleTrait = style;
    pDC.setStroke(Pinceau.CreePinceau(styleTrait));
    pDC.setColor(couleur);
    traceAngleDroit(pDC);
  }
  
  @Override
	public boolean confonduAvec(CElementBase p) {
    CMarqueAngleDroit pt;
    if (p.getClass() == this.getClass()) {
      pt = (CMarqueAngleDroit) p;
  		return (perp == pt.perp);
   }
    else return false;
	}
  
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    perp = (CDroitePerpendiculaire) (list.get(ind1,CDroitePerpendiculaire.class)); 
  }  
  /* Lecture depuis dans un flux java */  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    perp = (CDroitePerpendiculaire) (list.get(ind1,CDroitePerpendiculaire.class)); 
  }  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(perp);
    oups.writeInt(ind1);
  }  
}
