/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.geom.*;
import java.io.*;
import java.awt.Color;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;


public class CPointBaseEnt extends CPointAncetrePointsMobiles{
  public CRepere rep;
  public boolean absLibres; // True si les absisses sont libres. Sinon il y aura une abscisse mini
  // et une maxi
  public boolean ordLibres; // True si les ordonnes sont libres. Sinon il y aura une ordonne mini
  // et une maxi
  // Modification de la version 2.5 : les 4 valeurs suivantes deviennent dynamiques
  /*
  int absMin;
  int absMax;
  int ordMin;
  int ordMax;
   **/
  public CValeurAssocieeAVariable absMin;
  public CValeurAssocieeAVariable absMax;
  public CValeurAssocieeAVariable ordMin;
  public CValeurAssocieeAVariable ordMax;
  double absmin;
  double absmax;
  double ordmin;
  double ordmax;
  
  double abs;
  double ord;
  
  /** Creates a new instance of CPointBaseEnt */
  public CPointBaseEnt() {
  }
  // Constructeur pour clonage
  public CPointBaseEnt(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit, boolean masqueInit,
    String nomInit, int indiceTailleNomInit, int motifInit , boolean pointLibreInit,
    CRepere repInit,  boolean absLibresInit, boolean ordLibresInit, CValeurAssocieeAVariable absMinInit,
    CValeurAssocieeAVariable absMaxInit, CValeurAssocieeAVariable ordMinInit,
    CValeurAssocieeAVariable ordMaxInit, double absInit, double ordInit, boolean marquePourTraceInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
      dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit, motifInit, pointLibreInit, marquePourTraceInit);
    rep = repInit;
    absLibres = absLibresInit;
    ordLibres = ordLibresInit;
    absMin = absMinInit;
    absMax = absMaxInit;
    ordMin = ordMinInit;
    ordMax = ordMaxInit;
    abs = absInit;
    ord = ordInit;
  }
  
  @Override  
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(rep);
    int ind2 = listeSource.indexOf(impProto);
    CValeurAssocieeAVariable absMinClone = (CValeurAssocieeAVariable) absMin.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable absMaxClone = (CValeurAssocieeAVariable) absMax.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable ordMinClone = (CValeurAssocieeAVariable) ordMin.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable ordMaxClone = (CValeurAssocieeAVariable) ordMax.getClone(listeSource, listeCible);
    
    CElementBase ptelb = new CPointBaseEnt(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque,
      nom, indiceTailleNom, motif, pointLibre,  (CRepere)(listeCible.get(ind1, CRepere.class)),
      absLibres, ordLibres, absMinClone, absMaxClone, ordMinClone, ordMaxClone, abs, ord, marquePourTrace);
    return ptelb;
  }
  
  // Ajout version 4.8
  @Override
  public void initialisePourDependance() {
  	super.initialisePourDependance();
  	if (!absLibres) {
  		absMin.initialisePourDependance();
  		absMax.initialisePourDependance();
  	}
  	if (!ordLibres) {
	  	ordMin.initialisePourDependance();
	  	ordMax.initialisePourDependance();
  	}
  }

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    boolean res = super.dependDe(p) || rep.dependDe(p);
    if (!absLibres) res = res || absMin.dependDe(p) || absMax.dependDe(p);
    if (!ordLibres) res = res || ordMin.dependDe(p) || ordMax.dependDe(p);
    return memoriseDependance(res);
  }
    
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || rep.dependDePourBoucle(p);
  }
  
  @Override  
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = rep.existe;
    if (!existe) return;
    if (!absLibres) {
    	absMin.positionne(infoRandom, dimfen);
    	absMax.positionne(infoRandom, dimfen);
      existe = existe && absMin.existe() && absMax.existe();
      if (existe) {
        absmin = Math.floor(absMin.rendValeur(infoRandom) + 0.5);
        absmax = Math.floor(absMax.rendValeur(infoRandom) + 0.5);
        existe = (absmin <= absmax);
        if (existe) {
          if (abs < absmin) abs = absmin;
          else {
            if (abs > absmax) abs = absmax;
          }
        } else return;
      } else return;
    }
    if (!ordLibres) {
    	ordMin.positionne(infoRandom, dimfen);
    	ordMax.positionne(infoRandom, dimfen);
      existe = existe && ordMin.existe() && ordMax.existe();
      if (existe) {
        ordmin = Math.floor(ordMin.rendValeur(infoRandom) + 0.5);
        ordmax = Math.floor(ordMax.rendValeur(infoRandom) + 0.5);
        existe = (ordmin <= ordmax);
        if (existe) {
          if (ord < ordmin) ord = ordmin;
          else {
            if (ord > ordmax) ord = ordmax;
          }
        } else return;
      } else return;
    }    
    VecteurDirecteur vec = new VecteurDirecteur();
    vec.x = abs*rep.u.x + ord*rep.v.x;
    vec.y = abs*rep.u.y + ord*rep.v.y;
    x = rep.origine.x + vec.x;
    y = rep.origine.y + vec.y;
    super.positionne(infoRandom, dimfen);
  }
  // Teste si la position (xtest, ytest) (correspondant au point cliqu) est permise
  // si oui, renvoie dans pointr.x et pointr.y l'abscisse et l'ordonne entires correspondantes
  // dans le repre rep
  public boolean testDeplacement(DimensionFenetre dimfen, double xtest, double ytest,
    Point2D.Double pointr, Doub abscr) {
    double[] x1 = new double[4];
    double[] y1 = new double[4];
    double xr, yr;
    
    if (!testFenetre(dimfen, xtest, ytest)) return false; // Rajout version 4.2.4 car on pouvait sortir le point de la fentre
    double ux = rep.u.x;
    double uy = rep.u.y;
    double vx = rep.v.x;
    double vy = rep.v.y;
    double d = ux*vy-uy*vx;
    double ox = rep.origine.x;
    double oy = rep.origine.y;
    
    double x2 = xtest - ox;
    double y2 = ytest - oy;
    // On calcule les coordonnes dans le repre du point test.
    double ab = (vy*x2-vx*y2)/d;
    double or = (ux*y2-uy*x2)/d;
    // On calcule les coordonnes dans ce repre des quatre points  coordonnes
    // entires les plus proches
    double abint= Math.floor(ab);
    double orint = Math.floor(or);
    // On regarde d'abor quel est le point parmi les quatre qui est le plus proche.
    // S'il n'est pas hors-fentre, c'est le bon
    if ((ab-abint)<=0.5) xr = abint; else xr = abint+1;
    if ((or-orint)<=0.5) yr = orint; else yr = orint+1;
    VecteurDirecteur vec = new VecteurDirecteur();
    vec.x = xr*ux + yr*vx;
    vec.y = xr*uy + yr*vy;
    double xt = ox + vec.x;
    double yt = oy + vec.y;
    if (!testFenetre(dimfen,xt,yt)){
      // Le point le plus proche est hors-fentre
      x1[0] = abint;
      y1[0] = orint;
      x1[1] = abint+1;
      y1[1] = orint;
      x1[2] = abint+1;
      y1[2] = orint+1;
      x1[3] = abint;
      y1[3] = orint+1;
      double dis = 1; // La distance mini est au plus gale  1
      int ind = 0;
      for (int i=0; i<4; i++) {
        if (testFenetre(dimfen,x1[i],y1[i])) {
          double d1 = Kernel.distancePointPoint(ab,or,x1[i],y1[i]);
          if (d1<dis) {
            ind = i;
            dis = d1;
          }
        }
      }
      xr = x1[ind];
      yr = y1[ind];
    }
    if (!absLibres) {
      if (xr < absmin) xr = absmin;
      else {
        if (xr > absmax) xr = absmax;
      }
    }
    if (!ordLibres) {
      if (yr < ordmin) yr = ordmin;
      else {
        if (yr > ordmax) yr = ordmax;
      }
    }
    pointr.x = xr;
    pointr.y = yr;
    return true;
  }
  // PlaceEn place lengthpoint correspondant avec xn et yn comme abscisse et ordonne dans le
  // repre associ
  public void placeEn(double xn, double yn) {
    existe = true;
    abs = xn;
    ord = yn;
    //placeNom(xn, yn);
    // Diffrence par rapport au C++ : on ne peut pas appeler positionne de CPointAncetre
    horsEcran = !(testToile());
    positionneNom();
  }
  
  @Override
  public long getNature() {
    return NatureObjet.NPointBaseEnt;
  }
  
  @Override
  public String infoPourHistorique() {
    String ch = getString("chinfo139") + rep.nom();
    if (!absLibres) ch = ch + "\n" + getString("chinfo139Bis") + " [" + absMin.chaineInfo() + ";" + absMax.chaineInfo() + "]";
    if (!ordLibres) ch = ch + "\n" + getString("chinfo139Ter") + " [" + ordMin.chaineInfo() + ";" + ordMax.chaineInfo() + "]";
    return ch;
  }  
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    absMin = new CValeurAssocieeAVariable();
    absMax = new CValeurAssocieeAVariable();
    ordMin = new CValeurAssocieeAVariable();
    ordMax = new CValeurAssocieeAVariable();    
    int ind1 = inps.readCInt();
    rep = (CRepere) (list.get(ind1,CRepere.class));
    abs = inps.readCDouble();
    ord = inps.readCDouble();
    absLibres = inps.readBoolean();
    ordLibres = inps.readBoolean();
    if (nVersion == 1) {
      int d = inps.readCInt();
      absMin.donneValeur(d);
      d = inps.readInt();
      absMax.donneValeur(d);
      d = inps.readCInt();
      ordMin.donneValeur(d);
      d = inps.readCInt();
      ordMax.donneValeur(d);
    }
    else {
      if (!absLibres) {
        absMin.chargeDepuisFlux(inps, list);
        absMax.chargeDepuisFlux(inps, list);
      }
      else {
        absMin.donneValeur(-5);
        absMax.donneValeur(5);
      }
      if (!ordLibres) {
        ordMin.chargeDepuisFlux(inps, list);
        ordMax.chargeDepuisFlux(inps, list);
      }
      else {
        ordMin.donneValeur(-5);
        ordMax.donneValeur(5);
      }
    }
    nVersion = numeroVersion(); // On remet tout  zro pour la version java
  }

  @Override
  public void read ( MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    absMin = new CValeurAssocieeAVariable();
    absMax = new CValeurAssocieeAVariable();
    ordMin = new CValeurAssocieeAVariable();
    ordMax = new CValeurAssocieeAVariable();    
    int ind1 = inps.readInt();
    rep = (CRepere) (list.get(ind1,CRepere.class));
    abs = inps.readDouble();
    ord = inps.readDouble();
    absLibres = inps.readBoolean();
    ordLibres = inps.readBoolean();
    if (!absLibres) {
      absMin.read(inps, list);
      absMax.read(inps, list);
    }
    else {
      absMin.donneValeur(-5);
      absMax.donneValeur(5);
    }
    if (!ordLibres) {
      ordMin.read(inps, list);
      ordMax.read(inps, list);
    }
    else {
      ordMin.donneValeur(-5);
      ordMax.donneValeur(5);    
    }
  }
  
  @Override
  public void write ( MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(rep);
    oups.writeInt(ind1);
    oups.writeDouble(abs);
    oups.writeDouble(ord);
    oups.writeBoolean(absLibres);
    oups.writeBoolean(ordLibres);
    if (!absLibres) {
      absMin.write(oups, list);
      absMax.write(oups, list);
    }
    if (!ordLibres) {
      ordMin.write(oups, list);
      ordMax.write(oups, list);
    }
  }
  
}
