/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.geom.*;
import java.awt.Color;


import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;


public class CPointLieDroite extends CPointLie {
	private CDroiteAncetre droiteLiee;
  private Point2D.Double pointRes = new Point2D.Double();

  /** Creates new CPointLieDroite */
  public CPointLieDroite() {
  }
  
  // Constructeur pour clonage
  public CPointLieDroite(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit,
      boolean masqueInit, String nomInit, int indiceTailleNomInit,
      int motifInit, boolean pointLibreInit, double abscisseInit, CDroiteAncetre droiteLieeInit, boolean MarquePourTraceInit) {
     super (liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
       dec_xInit, dec_yInit, masqueInit, nomInit,
      indiceTailleNomInit, motifInit, pointLibreInit, abscisseInit, MarquePourTraceInit);
     droiteLiee = droiteLieeInit;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(droiteLiee);
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CPointLieDroite(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom,
      indiceTailleNom, motif, pointLibre, abscisse, (CDroiteAncetre)(listeCible.get(ind1, CDroiteAncetre.class)), marquePourTrace);
    return ptelb;
  }
  

  @Override
  public void ajouteAntecedents(CListeObjets liste) {
    liste.add(droiteLiee);
  }  
  
  @Override
  public String infoPourHistorique() {
    return getString("chinfo2") + " " + droiteLiee.complementInfo();
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || droiteLiee.dependDe(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || droiteLiee.dependDePourBoucle(p);
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
		existe = droiteLiee.existe;
		if (!existe) return;
		double xn = droiteLiee.point_x + abscisse*droiteLiee.vecteur.x;
		double yn = droiteLiee.point_y + abscisse*droiteLiee.vecteur.y;
		super.placeEn(xn,yn);
		super.positionne(infoRandom, dimfen); // Ajout version 4.6.4
  }

  // La procdure suivante teste si le dplacement du point correspondant  la position
  // (xtest, ytest)  de la souris est possible et si oui renvoyant dans pointr les
  // nouvelles coordonnes et dans pointAbs.x l'abscisse pour un point li

  @Override
  public boolean testDeplacement(DimensionFenetre dimfen, double xtest, double ytest,
        Point2D.Double pointr, Doub abscr) {
          
    Kernel.projetteOrtho(xtest,ytest,droiteLiee.point_x,droiteLiee.point_y,droiteLiee.vecteur,pointRes);
		boolean valable = (droiteLiee.appartientA(pointRes.x,pointRes.y))
      && (dimfen.dansFenetre(pointRes.x,pointRes.y));
		if (!valable) return false;
    pointr.setLocation(pointRes);
		double absx = Math.abs(droiteLiee.vecteur.x);
		double absy = Math.abs(droiteLiee.vecteur.y);
		if ((absx == 0) && (absy == 0))
		{
			valable = false;
		}
		else
		{
			if (absx > absy)
			 abscr.setValue((pointr.x - droiteLiee.point_x)/droiteLiee.vecteur.x);
			else
			 abscr.setValue((pointr.y - droiteLiee.point_y)/droiteLiee.vecteur.y);
		}
		return valable;
  }

  @Override
  public double abscisseMaximale() {
  	return droiteLiee.abscisseMaximale();
  }

  @Override
  public double abscisseMinimale() {
  	return droiteLiee.abscisseMinimale();
  }

  @Override
	public
  CElementBase lieA() {
    return droiteLiee;
  }
  
  @Override
  public boolean confonduAvec(CElementBase p) {
  	boolean res = super.confonduAvec(p);
   	if (!res) return false;
  	CPointLieDroite pt = (CPointLieDroite) p;
  	return (droiteLiee == pt.droiteLiee);
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    droiteLiee = (CDroiteAncetre) (list.get(ind1,CDroiteAncetre.class));
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    droiteLiee = (CDroiteAncetre) (list.get(ind1,CDroiteAncetre.class));
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(droiteLiee);
    oups.writeInt(ind1);
  }

  @Override
  public boolean appartientParDefinition(CDroiteAncetre droite) {
    return super.appartientParDefinition(droite) || (droite == droiteLiee);
  }
  
}
