/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.Color;
import java.io.IOException;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import types.DimensionFenetre;

public class CPointParMultVect extends CPointAncetre {
	public CPointAncetre antecedent;
	public CVecteur vecteur;
	public CValeurAssocieeAVariable coef;
	
	/**
	 * Constructeur par dfaut
	 */
	public CPointParMultVect() {
	}
	
  /** Constructeur pour clonage */
  public CPointParMultVect(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit, boolean masqueInit, String nomInit, int indiceTailleNomInit,
      int motifInit , CPointAncetre antecedentInit, CVecteur vecteurInit, CValeurAssocieeAVariable coefInit, boolean marquePourTraceInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
          dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit, motifInit, marquePourTraceInit);
        antecedent = antecedentInit;
        vecteur = vecteurInit;
        coef = coefInit;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(antecedent);
    int ind2 = listeSource.indexOf(vecteur); 
    int ind3 = listeSource.indexOf(impProto);
    CValeurAssocieeAVariable coefClone = (CValeurAssocieeAVariable) coef.getClone(listeSource, listeCible);
    
    CElementBase ptelb = new CPointParMultVect(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom,
        indiceTailleNom, motif, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
        (CVecteur)listeCible.get(ind2, CVecteur.class), coefClone, marquePourTrace);
    return ptelb;
  }  

  // Ajout version 4.8
  @Override
  public void initialisePourDependance() {
  	super.initialisePourDependance();
  	coef.initialisePourDependance();
  }

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || antecedent.dependDe(p) || vecteur.dependDe(p) || coef.dependDe(p));
  }
    
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || antecedent.dependDePourBoucle(p) || vecteur.dependDePourBoucle(p)) || coef.dependDePourBoucle(p);
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	coef.positionne(infoRandom, dimfen);
		existe = antecedent.existe && vecteur.existe && coef.existe();
		if (!existe) return;
		double k = coef.rendValeur(infoRandom);
		super.placeEn(antecedent.x + k*(vecteur.x2 - vecteur.x1), 
				antecedent.y + k*(vecteur.y2 - vecteur.y1));
		super.positionne(infoRandom, dimfen); // dimfen Ajout version 4.6.4
	}
  
  @Override
  public boolean confonduAvec(CElementBase p) {
    CPointParMultVect pt;
    if (p.getClass() == this.getClass()) {
      pt = (CPointParMultVect) p;
   		return (antecedent == pt.antecedent) && (vecteur == pt.vecteur) && coef.confonduAvec(pt.coef);
    }
    else return false;
  }
  
  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (antecedent == ancienPoint) antecedent = nouveauPoint;
  }
     
   /** Chargement depuis un flux java */
   public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
     super.read(inps, list);
     int ind1 = inps.readInt();
     int ind2 = inps.readInt();
     antecedent = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
     vecteur = (CVecteur) (list.get(ind2,CVecteur.class));
     coef = new CValeurAssocieeAVariable();
     coef.read(inps, list);
   }
   
   /** Enregistrement dans un flux java */
   public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
     super.write(oups, list);
     int ind1 = list.indexOf(antecedent);
     oups.writeInt(ind1);
     int ind2 = list.indexOf(vecteur);
     oups.writeInt(ind2);
     coef.write(oups, list);
   }

}
