/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.Color;
import java.io.IOException;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import types.DimensionFenetre;

/**
 * Objet rajout pour la version 1.6
 * Crer un point image d'un autre par la translation dont le vecteur est la somme des
 * deux vecteurs cliqus.
 * @author Yves
 *
 */
public class CPointParSommeVect extends CPointAncetre {
	private CPointAncetre antecedent;
	private CVecteur vecteur1;
	private CVecteur vecteur2;
	
	/**
	 * Constructeur par dfaut
	 */
	public CPointParSommeVect() {
		
	}
	
  /** Constructeur pour clonage */
  public CPointParSommeVect(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit, boolean masqueInit, String nomInit, int indiceTailleNomInit,
      int motifInit , CPointAncetre antecedentInit, CVecteur vecteur1Init, CVecteur vecteur2Init, boolean marquePourTraceInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
          dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit, motifInit, marquePourTraceInit);
        antecedent = antecedentInit;
        vecteur1 = vecteur1Init;
        vecteur2 = vecteur2Init;
  }  	

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(antecedent);
    int ind2 = listeSource.indexOf(vecteur1); 
    int ind3 = listeSource.indexOf(vecteur2); 
    int ind4 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CPointParSommeVect(listeCible, (CImplementationProto)(listeCible.get(ind4, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom,
        indiceTailleNom, motif, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
        (CVecteur)listeCible.get(ind2, CVecteur.class), (CVecteur)listeCible.get(ind3, CVecteur.class), marquePourTrace);
    return ptelb;
  }
   
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || antecedent.dependDe(p) || vecteur1.dependDe(p) || vecteur2.dependDe(p));
  }
    
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || antecedent.dependDePourBoucle(p) || vecteur1.dependDePourBoucle(p)) || vecteur2.dependDePourBoucle(p);
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	
		existe = antecedent.existe && vecteur1.existe && vecteur2.existe;
		if (!existe) return;
		super.placeEn(antecedent.x + vecteur1.x2 - vecteur1.x1 + vecteur2.x2 - vecteur2.x1, 
				antecedent.y + vecteur1.y2 - vecteur1.y1 + vecteur2.y2 - vecteur2.y1);
		super.positionne(infoRandom, dimfen); // Ajout version 4.6.4
	}
  
  @Override
  public boolean confonduAvec(CElementBase p) {
    CPointParSommeVect pt;
    if (p.getClass() == this.getClass()) {
      pt = (CPointParSommeVect) p;
      return (antecedent.confonduAvec(pt.antecedent) && ((vecteur1.confonduAvec(pt.vecteur1) && vecteur2.confonduAvec(pt.vecteur2))
      		|| (vecteur1.confonduAvec(pt.vecteur2) && vecteur2.confonduAvec(pt.vecteur1))));
    } else return false;
  }
  
  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (antecedent == ancienPoint) antecedent = nouveauPoint;
  }
    
  /** Chargement depuis un flux java */
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    int ind3 = inps.readInt();
    antecedent = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    vecteur1 = (CVecteur) (list.get(ind2,CVecteur.class));
    vecteur2 = (CVecteur) (list.get(ind3,CVecteur.class));
  }
  
  /** Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(antecedent);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(vecteur1);
    oups.writeInt(ind2);
    int ind3 = list.indexOf(vecteur2);
    oups.writeInt(ind3);
  }
  
}
