/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package objets;

import java.io.IOException;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import types.DimensionFenetre;
import types.NatureCalcul;

public class CTestEquivalence extends CCalculAncetre {
	public CCalcul calcul1;
	public CCalcul calcul2;
	public CCalculBase calculeq1;
	public CCalculBase calculeq2;
	public boolean remplacementValeurs1;
	public boolean remplacementValeurs2;
	protected double resultat;
	protected boolean dejaPositionne;
	// Ajout version 4.9.8 : un test d'existence. Le test d'quivalence ne sera valu que si ce test renvoie 1.
	public CValeurAssocieeAVariable test;
	
	/**
	 * Constructeur
	 */
	public CTestEquivalence() {
		existe = true;
		dejaPositionne = false;
	}
	
  // Version 4.9.8. On change le numro de version.
  // On rajoute un membre test. Le test d'quivalence ne sera valu que si test est true.
  @Override
  public int numeroVersion() {
  	return 2;
  }  

  public CTestEquivalence(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      String nomCalculInit, CCalcul calcul1Init, CCalcul calcul2Init, boolean remplacementValeursInit1,
      boolean remplacementValeursInit2, CValeurAssocieeAVariable testInit) {
      super(liste, impProtoInit, estElementFinalInit, nomCalculInit);
      calcul1 = calcul1Init;
      calcul2 = calcul2Init;
      remplacementValeurs1 = remplacementValeursInit1;
      remplacementValeurs2 = remplacementValeursInit2;
      existe = true;
      dejaPositionne = false;
      test = testInit;
    }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(impProto);
    int ind2 = listeSource.indexOf(calcul1);
    int ind3 = listeSource.indexOf(calcul2);
    CValeurAssocieeAVariable testClone = (CValeurAssocieeAVariable) test.getClone(listeSource, listeCible);
    return new CTestEquivalence(listeCible, (CImplementationProto)(listeCible.get(ind1, CImplementationProto.class)),
      estElementFinal, nomCalcul, (CCalcul)(listeCible.get(ind2, CCalcul.class)),
      (CCalcul)(listeCible.get(ind3, CCalcul.class)), remplacementValeurs1, remplacementValeurs2, testClone);
  }
  
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NTestEquivalence;
  }
  
  
  public double rendValeur(boolean infoRandom) {
    return resultat;
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    // Existe toujours gal  true pour cet objet. Fini  partir version 4.9.8
  	// Ajout version 4.9.8
  	test.positionne(infoRandom, dimfen);
  	if (!test.existe()) {
  		resultat = 0;
  		return;
  	}
  	if (test.rendValeur(false) != 1) {
  		resultat = 0;
  		return;
  	}
  	//
  	if (!dejaPositionne) { 
  		if (remplacementValeurs1) calculeq1 = calcul1.calcul.calculAvecValeursRemplacees().calculNormalise(true, true);
  		else calculeq1 = calcul1.calcul.calculNormalise(remplacementValeurs1, true);
  		if (remplacementValeurs2) calculeq2 = calcul2.calcul.calculAvecValeursRemplacees().calculNormalise(true, true);
  		else calculeq2 = calcul2.calcul.calculNormalise(remplacementValeurs2, true);
  		dejaPositionne = true; // Ajout version 4.7.2
  	}
    if (!calcul1.existe || !calcul2.existe) {
    	resultat = 0;
    	return;
    }
    if (calculeq1.estEquivalentA(calculeq2)) resultat = 1; else resultat = 0;
  }

  @Override
  // Ajout version 4.9.3. Utilis dans les sommes et produits indics
  public void positionneFull(boolean infoRandom, DimensionFenetre dimfen) {
    // Existe toujours gal  true pour cet objet. Fini pour version 4.9.8
  	// Ajout version 4.9.8
  	test.positionne(infoRandom, dimfen);
  	if (!test.existe()) {
  		resultat = 0;
  		return;
  	}
  	if (test.rendValeur(false) != 1) {
  		resultat = 0;
  		return;
  	}
  	//
		if (remplacementValeurs1) calculeq1 = calcul1.calcul.calculAvecValeursRemplacees().calculNormalise(true, true);
		else calculeq1 = calcul1.calcul.calculNormalise(remplacementValeurs1, true);
		if (remplacementValeurs2) calculeq2 = calcul2.calcul.calculAvecValeursRemplacees().calculNormalise(true, true);
		else calculeq2 = calcul2.calcul.calculNormalise(remplacementValeurs2, true);
		// dejaPositionne = false; // Modifi version 4.9.6
		dejaPositionne = true;
    if (!calcul1.existe || !calcul2.existe) {
    	resultat = 0;
    	return;
    }
    if (calculeq1.estEquivalentA(calculeq2)) resultat = 1; else resultat = 0;
  }

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || calcul1.dependDe(p) || calcul2.dependDe(p) || test.dependDe(p));
  }
    
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || calcul1.dependDePourBoucle(p) || calcul2.dependDePourBoucle(p) || test.dependDePourBoucle(p);
  }

  @Override
  public boolean modifiableParMenu() {
  	return !estElementFinal;
  }
   
  public void metAJour() {
  	 dejaPositionne = false;
  }

	 @Override
	 public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
	   super.read(inps, list);
	   int ind1 = inps.readInt();
	   calcul1 = (CCalcul) (list.get(ind1,CCalcul.class));
	   int ind2 = inps.readInt();
	   calcul2 = (CCalcul) (list.get(ind2,CCalcul.class));
	   remplacementValeurs1 = inps.readBoolean();
	   remplacementValeurs2 = inps.readBoolean();
	   // calculeq1 = calcul1.calcul.calculNormalise(remplacementValeurs);
	   // calculeq2 = calcul2.calcul.calculNormalise(remplacementValeurs);
	   // Ajout version 4.9.8
	   test = new CValeurAssocieeAVariable();
	   if (nVersion < 2) test.donneValeur(1);
	   else test.read(inps, list);
	 }

	@Override
	public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
	  super.write(oups, list);
	  int ind1 = list.indexOf(calcul1);
	  oups.writeInt(ind1);
	  int ind2 = list.indexOf(calcul2);
	  oups.writeInt(ind2);
	  oups.writeBoolean(remplacementValeurs1);
	  oups.writeBoolean(remplacementValeurs2);
	  // Ajout version 4.9.8
	  test.write(oups, list);
	}          

}
