/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;
import java.io.IOException;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import types.Complexe;
import types.FonteNom;

// Pour la version 3.0 on autorise les fonctions de deux variables
// Une variable formelle est donc caractrise par son indice de variable
// valant 1 ou 2 suivant que c'est la premire variable ou la seconde
// Donc nouveau numro de version 2.

public class CVariableFormelle extends CCalculBase {

	int indvar;		// Nouveau membre pour la version 3.0 : indice de la variable formelle
								// 1 si c'est une variable formelle d'une fonction d'une seule variable
								// Sinon 1 ou 2 pour une fonction de deux variables
	
  /** Creates new CVariableFormelle */
  public CVariableFormelle() {
  }

  public CVariableFormelle(CListeObjets liste, int indvarInit) {
    super(liste);
    indvar = indvarInit;
  }
  
  @Override
  protected long nature() {
  	return natVariableFormelle;
  }  
  
  // A revoir version 3.0
  @Override
  public double resultatFonction(boolean infoRandom, double valeurParametre) {
    return valeurParametre;
  }
  
  // A revoir version 3.0
  @Override
  public double resultatFonction(boolean infoRandom, double[] valeurParametre) {
    return valeurParametre[indvar];
  }
  
  
  // A revoir version 3.0
  // Rajout pour la version 2.0
  @Override
  public void  resultatFonctionComplexe(boolean infoRandom, Complexe valeurParametre, Complexe zRes) {
    zRes.x = valeurParametre.x;
    zRes.y = valeurParametre.y;
  }

  @Override
  public void  resultatFonctionComplexe(boolean infoRandom, Complexe[] valeurParametre, Complexe zRes) {
    zRes.x = valeurParametre[indvar].x;
    zRes.y = valeurParametre[indvar].y;
  }
  
  CCalculBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    CCalculBase ptcalc = new CVariableFormelle(listeCible, indvar);
    return ptcalc;
  }
  
  /**
   * Modifi version 3.0 : Utilis dans le calcul formel de drives
   * ind reprsente l'indice de variable cherch (0 pour premire variable)
   */
  public boolean  dependDeVariable(int ind) {
    return (indvar == ind);
  }
  
  public CCalculBase getCopie() {
  	return new CVariableFormelle(listeProprietaire, indvar);
  }
  
  // A revoir version 3.0
  @Override
	public String chaineCalcul(String[] varFor) {
		return varFor[indvar];	
	}
	@Override
  public String chaineLatex(String[] varFor) {
		String ch = chaineCalcul(varFor);
		if (FonteNom.estNomGrec(ch)) return " \\"+ ch + " ";
		else return ch;
	}
  
  @Override
	public boolean estConstant() {
  	// Erreur corrige version 4.2. Les fonctions ne doivent pas tre considres comme des constantes.
		// return true;
  	return false;
	}
  
  /**
   * A partir de la version 3.0, changement de version (version 1 -> 2) car on peut avoir des fonctions de plusieurs variables
   */
  @Override
  public int numeroVersion() {
  	return 2;
  }  

  // A partir de la version 3.0, une variable formelle possde un membre devant tre enregistr dans le flux.
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    if (nVersion == 1) indvar = 0;
    else indvar = inps.readInt();
  }
  
  // Ajout version 4.6
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeInt(indvar);
  }
}
